@@ -185,8 +185,13 @@ def parse_rfc822(path):
         # completely ignore ^# as that breaks continuation lines
         if l.startswith('#'):
             continue
-        # filter out comments which don't start on first column
-        l = l.split('#', 1)[0]
+        # filter out comments which don't start on first column (Debian
+        # #743174); entirely remove line if all that's left is whitespace, as
+        # that again breaks continuation lines
+        if '#' in l:
+            l = l.split('#', 1)[0]
+            if not l.strip():
+                continue
         lines.append(l)
     f.close()
 
@@ -228,19 +233,33 @@ def _debian_packages_from_source(srcdir):
     return packages
 
 
-def _debian_build_deps_from_source(srcdir):
-    deps = []
+def _debian_build_deps_from_source(srcdir, testbed_arch):
+    deps = ''
     for st in parse_rfc822(os.path.join(srcdir, 'debian/control')):
         if 'Build-depends' in st:
-            for d in st['Build-depends'].split(','):
-                dp = d.strip()
-                if dp:
-                    deps.append(dp)
+            deps += st['Build-depends']
         if 'Build-depends-indep' in st:
-            for d in st['Build-depends-indep'].split(','):
-                dp = d.strip()
-                if dp:
-                    deps.append(dp)
+            deps += ', ' + st['Build-depends-indep']
+
+    # resolve arch specific dependencies and build profiles
+    perl = subprocess.Popen(['perl', '-'], stdin=subprocess.PIPE,
+                            stdout=subprocess.PIPE)
+    code = '''use Dpkg::Deps;
+              $supports_profiles = ($Dpkg::Deps::VERSION gt '1.04');
+              $dep = deps_parse('%s', reduce_arch => 1,
+                                reduce_profiles => $supports_profiles,
+                                build_dep => 1, host_arch => '%s');
+              $out = $dep->output();
+              # fall back to ignoring build profiles
+              $out =~ s/ <[^ >]+>//g if (!$supports_profiles);
+              print $out, "\\n";
+              ''' % (deps, testbed_arch)
+    deps = perl.communicate(code.encode('UTF-8'))[0].decode('UTF-8').strip()
+    if perl.returncode != 0:
+        raise InvalidControl('source', 'Invalid build dependencies')
+
+    deps = [d.strip() for d in deps.split(',')]
+
     # @builddeps@ should always imply build-essential
     deps.append('build-essential')
     return deps
@@ -272,7 +291,7 @@ def _debian_check_dep(testname, dep):
             pass
 
 
-def _parse_debian_depends(testname, dep_str, srcdir):
+def _parse_debian_depends(testname, dep_str, srcdir, testbed_arch):
     '''Parse Depends: line in a Debian package
 
     Split dependencies (comma separated), validate their syntax, and expand @
@@ -293,7 +312,7 @@ def _parse_debian_depends(testname, dep_str, srcdir):
                 adtlog.debug('synthesised dependency %s' % d)
                 deps.append(d)
         elif alt_group_str == '@builddeps@':
-            for d in _debian_build_deps_from_source(srcdir):
+            for d in _debian_build_deps_from_source(srcdir, testbed_arch):
                 adtlog.debug('synthesised dependency %s' % d)
                 deps.append(d)
         else:
@@ -329,7 +348,8 @@ def _autodep8(srcdir):
     return None
 
 
-def parse_debian_source(srcdir, testbed_caps, control_path=None):
+def parse_debian_source(srcdir, testbed_caps, testbed_arch, control_path=None,
+                        auto_control=True):
     '''Parse test descriptions from a Debian DEP-8 source dir
 
     You can specify an alternative path for the control file (default:
@@ -348,11 +368,15 @@ def parse_debian_source(srcdir, testbed_caps, control_path=None):
     if not control_path:
         control_path = os.path.join(srcdir, 'debian', 'tests', 'control')
 
-    if not os.path.exists(control_path):
-        control = _autodep8(srcdir)
-        if control is None:
-            return ([], False)
-        control_path = control.name
+        if not os.path.exists(control_path):
+            if auto_control:
+                control = _autodep8(srcdir)
+                if control is None:
+                    return ([], False)
+                control_path = control.name
+            else:
+                adtlog.debug('auto_control is disabled, no tests')
+                return ([], False)
 
     for record in parse_rfc822(control_path):
         command = None
@@ -365,7 +389,8 @@ def parse_debian_source(srcdir, testbed_caps, control_path=None):
                 test_names = record['Tests'].replace(',', ' ').split()
                 depends = _parse_debian_depends(test_names[0],
                                                 record.get('Depends', '@'),
-                                                srcdir)
+                                                srcdir,
+                                                testbed_arch)
                 if 'Test-command' in record:
                     raise InvalidControl('*', 'Only one of "Tests" or '
                                          '"Test-Command" may be given')
@@ -381,7 +406,8 @@ def parse_debian_source(srcdir, testbed_caps, control_path=None):
                 command = record['Test-command']
                 depends = _parse_debian_depends(command,
                                                 record.get('Depends', '@'),
-                                                srcdir)
+                                                srcdir,
+                                                testbed_arch)
                 command_counter += 1
                 name = 'command%i' % command_counter
                 _debian_check_unknown_fields(name, record)
@@ -461,7 +487,8 @@ def parse_click_manifest(manifest, testbed_caps, clickdeps, use_installed,
 
         # autopilot special case: dict with extra depends
         if 'autopilot_module' in desc:
-            desc['command'] = 'PYTHONPATH=app/tests/autopilot:tests/autopilot:$PYTHONPATH ' \
+            desc['command'] = \
+                'PYTHONPATH=app/tests/autopilot:tests/autopilot:$PYTHONPATH '\
                 'python3 -m autopilot.run run -v -f subunit -o ' \
                 '$ADT_ARTIFACTS/%s.subunit ' % name + os.environ.get(
                     'ADT_AUTOPILOT_MODULE', desc['autopilot_module'])
